import base64


def generate_basic_auth(username, password):
    basic_auth = f'{username}:{password}'
    basic_auth_creds = bytes(basic_auth, encoding='utf-8')
    creds = str(base64.b64encode(basic_auth_creds), encoding='utf-8')
    return f'basic {creds}'



"""
    Create or update Django super user with a unusable password

    A "unusable password" because it's not needed while auth via SSOwat ;)

    Can be called e.g.:
        ./manage.py create_superuser --username="bar" --email="foo@bar.tld"
"""


import sys

from django.contrib.auth import get_user_model
from django.core.management import BaseCommand


class Command(BaseCommand):
    help = 'Create or update Django super user with a unusable password (auth via SSOwat)'

    def add_arguments(self, parser):
        parser.add_argument(
            "--username",
            action="store",
            required=True,
        )
        parser.add_argument(
            "--email",
            action="store",
            default=None,
        )

    def handle(self, *args, **options):
        username = options['username']
        email = options['email']

        User = get_user_model()
        user = User.objects.filter(username=username).first()
        if user:
            self.stderr.write(f'Update existing user "{user}" and set his password.')
            user.is_active = True
            user.is_staff = True
            user.is_superuser = True
            if email:
                user.email = email
        else:
            print(f'Create new super user "{username}"', file=sys.stderr)
            user = User.objects.create_superuser(username=username, email=email, password=None)

        user.set_unusable_password()
        user.save()



import logging
from functools import lru_cache

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from django.utils.module_loading import import_string


logger = logging.getLogger(__name__)


UserModel = get_user_model()


@lru_cache(maxsize=None)
def get_setup_user_func():
    setup_user_func = import_string(settings.YNH_SETUP_USER)
    assert callable(setup_user_func)
    return setup_user_func


def call_setup_user(user):
    """
    Hook for the YunoHost package application to setup a Django user.
    Call function defined in settings.YNH_SETUP_USER

    called via:
        * SSOwatUserBackend after a new user was created
        * SSOwatRemoteUserMiddleware on login request
    """
    old_pk = user.pk

    setup_user_func = get_setup_user_func()
    logger.debug('Call "%s" for user "%s"', settings.YNH_SETUP_USER, user)

    user = setup_user_func(user=user)

    assert isinstance(user, UserModel)
    assert user.pk == old_pk

    return user


def update_user_profile(request, user):
    """
    Update existing user information:
     * Email
     * First / Last name

    Called via:
     * SSOwatUserBackend after a new user was created
     * SSOwatRemoteUserMiddleware on login request
    """
    update_fields = []

    if user.is_authenticated and not user.has_usable_password():
        # Empty password is not valid, so we can't save the model, because of full_clean() call
        logger.info('Set unusable password for user: %s', user)
        user.set_unusable_password()
        update_fields.append('password')

    email = request.META.get('HTTP_EMAIL')
    if email and user.email != email:
        logger.info('Update email: %r -> %r', user.email, email)
        user.email = email
        update_fields.append('email')

    raw_username = request.META.get('HTTP_NAME')
    if raw_username:
        if ' ' in raw_username:
            first_name, last_name = raw_username.split(' ', 1)
        else:
            first_name = ''
            last_name = raw_username

        if user.first_name != first_name:
            logger.info('Update first name: %r -> %r', user.first_name, first_name)
            user.first_name = first_name
            update_fields.append('first_name')

        if user.last_name != last_name:
            logger.info('Update last name: %r -> %r', user.last_name, last_name)
            user.last_name = last_name
            update_fields.append('last_name')

    if update_fields:
        try:
            user.full_clean()
        except ValidationError:
            logger.exception('Can not update user: %s', user)
        else:
            user.save(update_fields=update_fields)

    return user



"""
    Helper to create a random string for settings.SECRET_KEY

    SECURITY WARNING: keep the secret key used in production secret!
"""
import logging
from pathlib import Path
from secrets import token_urlsafe


logger = logging.getLogger(__name__)


def get_or_create_secret(secret_file):
    assert isinstance(secret_file, Path)
    assert secret_file.parent.is_dir, f'Directory does not exists: {secret_file.parent}'

    if not secret_file.is_file():
        logger.info('Generate %s', secret_file)
        secret_file.open('w').write(token_urlsafe(128))

    with secret_file.open('r') as f:
        return f.read()



from pathlib import Path


def assert_is_dir(dir_path):
    assert isinstance(dir_path, Path)
    assert dir_path.is_dir, f'Directory does not exists: {dir_path}'


def assert_is_file(file_path):
    assert isinstance(file_path, Path)
    assert file_path.is_file, f'File not found: {file_path}'


def is_relative_to(p, other):
    """
    Path.is_relative_to() is new in Python 3.9
    """
    p = Path(p)
    other = Path(other)
    try:
        p.relative_to(other)
    except ValueError:
        return False
    else:
        return True



import base64
import logging

from axes.exceptions import AxesBackendPermissionDenied
from django.conf import settings
from django.contrib import auth
from django.contrib.auth import get_user_model
from django.contrib.auth.middleware import RemoteUserMiddleware

from django_ynh.sso_auth.user_profile import call_setup_user, update_user_profile


logger = logging.getLogger(__name__)


UserModel = get_user_model()


class SSOwatRemoteUserMiddleware(RemoteUserMiddleware):
    """
    Middleware to login a user via HTTP_REMOTE_USER header.
    Use Django Axes if something is wrong.
    Update exising user informations.
    """

    header = 'HTTP_REMOTE_USER'
    force_logout_if_no_header = True

    def process_request(self, request):
        # Keep the information if the user is already logged in
        was_authenticated = request.user.is_authenticated

        super().process_request(request)  # login remote user

        user = request.user

        if not user.is_authenticated:
            logger.debug('Not logged in -> nothing to verify here')
            return

        # Check SSOwat cookie informations:
        try:
            username = request.COOKIES['SSOwAuthUser']
        except KeyError:
            logger.error('SSOwAuthUser cookie missing!')

            if settings.DEBUG:
                # e.g.: local test can't set a Cookie easily
                logger.warning('Ignore error, because settings.DEBUG is on!')
            else:
                # emits a signal indicating user login failed, which is processed by
                # axes.signals.log_user_login_failed which logs and flags the failed request.
                raise AxesBackendPermissionDenied('Cookie missing')
        else:
            logger.info('SSOwat username from cookies: %r', username)
            if username != user.username:
                raise AxesBackendPermissionDenied('Wrong username')

        # Compare with HTTP_AUTH_USER
        try:
            username = request.META['HTTP_AUTH_USER']
        except KeyError:
            logger.error('HTTP_AUTH_USER missing!')
            raise AxesBackendPermissionDenied('No HTTP_AUTH_USER')

        if username != user.username:
            raise AxesBackendPermissionDenied('Wrong HTTP_AUTH_USER username')

        # Also check 'HTTP_AUTHORIZATION', but only the username ;)
        try:
            authorization = request.META['HTTP_AUTHORIZATION']
        except KeyError:
            logger.error('HTTP_AUTHORIZATION missing!')
            raise AxesBackendPermissionDenied('No HTTP_AUTHORIZATION')

        scheme, creds = authorization.split(' ', 1)
        if scheme.lower() != 'basic':
            logger.error('HTTP_AUTHORIZATION with %r not supported', scheme)
            raise AxesBackendPermissionDenied('HTTP_AUTHORIZATION scheme not supported')

        creds = str(base64.b64decode(creds), encoding='utf-8')
        username = creds.split(':', 1)[0]
        if username != user.username:
            raise AxesBackendPermissionDenied('Wrong HTTP_AUTHORIZATION username')

        if not was_authenticated:
            # First request, after login -> update user informations
            logger.info('Remote user "%s" was logged in', user)
            user = update_user_profile(request, user)

            user = call_setup_user(user=user)
            assert isinstance(user, UserModel)

            # persist user in the session
            request.user = user
            auth.login(request, user)



import os
import sys
from pathlib import Path

from django_ynh.local_test import create_local_test
from django_ynh.path_utils import assert_is_dir, assert_is_file


def run_pytest(django_settings_path, destination):
    """
    1. Generate "local test installation"
    2. Run pytest against generated sources
    """
    assert_is_file(django_settings_path)

    conf_path = django_settings_path.parent
    base_path = conf_path.parent
    test_path = Path(base_path / 'tests')

    assert_is_dir(test_path)

    final_home_path = create_local_test(
        django_settings_path=django_settings_path,
        destination=destination,
        runserver=False,
    )
    django_settings_name = django_settings_path.stem
    os.environ['DJANGO_SETTINGS_MODULE'] = django_settings_name
    print(f'DJANGO_SETTINGS_MODULE={django_settings_name}')

    sys.path.insert(0, str(final_home_path))

    import pytest

    # collect only project tests and pass existing pytest arguments:
    sys.argv = [__file__, str(test_path)] + sys.argv[1:]

    raise SystemExit(pytest.console_main())



"""
    Create a YunoHost package local test
"""
import argparse
import os
import shlex
import subprocess
import sys
from pathlib import Path

from django_ynh.path_utils import assert_is_dir, assert_is_file
from django_ynh.test_utils import generate_basic_auth


def verbose_check_call(command, verbose=True, **kwargs):
    """ 'verbose' version of subprocess.check_call() """
    if verbose:
        print('_' * 100)
        msg = f'Call: {command!r}'
        verbose_kwargs = ', '.join(f'{k}={v!r}' for k, v in sorted(kwargs.items()))
        if verbose_kwargs:
            msg += f' (kwargs: {verbose_kwargs})'
        print(f'{msg}\n', flush=True)

    env = os.environ.copy()
    env['PYTHONUNBUFFERED'] = '1'

    popenargs = shlex.split(command)
    subprocess.check_call(popenargs, universal_newlines=True, env=env, **kwargs)


def call_manage_py(final_home_path, args):
    verbose_check_call(
        command=f'{sys.executable} manage.py {args}',
        cwd=final_home_path,
    )


def copy_patch(src_file, replaces, final_home_path):
    dst_file = final_home_path / src_file.name
    print(f'{src_file} -> {dst_file}')

    with src_file.open('r') as f:
        content = f.read()

    if replaces:
        for old, new in replaces.items():
            if old in content:
                print(f' * Replace "{old}" -> "{new}"')
                content = content.replace(old, new)

    with dst_file.open('w') as f:
        f.write(content)


def create_local_test(django_settings_path, destination, runserver=False):
    django_settings_path = django_settings_path.resolve()
    assert_is_file(django_settings_path)

    django_settings_name = django_settings_path.stem

    conf_path = django_settings_path.parent

    project_name = conf_path.parent.name

    assert isinstance(destination, Path)
    destination = destination.resolve()
    if not destination.is_dir():
        destination.mkdir(parents=False)

    assert_is_dir(destination)

    final_home_path = destination / 'opt_yunohost'
    final_www_path = destination / 'var_www'
    log_file = destination / f'var_log_{project_name}.log'

    REPLACES = {
        '__FINAL_HOME_PATH__': str(final_home_path),
        '__FINAL_WWW_PATH__': str(final_www_path),
        '__LOG_FILE__': str(log_file),
        '__PATH_URL__': 'app_path',
        '__DOMAIN__': '127.0.0.1',
        'django.db.backends.postgresql': 'django.db.backends.sqlite3',
        "'NAME': '__APP__',": f"'NAME': '{destination / 'test_db.sqlite'}',",
        'django_redis.cache.RedisCache': 'django.core.cache.backends.dummy.DummyCache',
        # Just use the default logging setup from django_ynh project:
        'LOGGING = {': 'HACKED_DEACTIVATED_LOGGING = {',
    }

    for p in (final_home_path, final_www_path):
        if p.is_dir():
            print(f'Already exists: "{p}", ok.')
        else:
            p.mkdir(parents=True, exist_ok=True)

    log_file.touch(exist_ok=True)

    for src_file in conf_path.glob('*.py'):
        copy_patch(src_file=src_file, replaces=REPLACES, final_home_path=final_home_path)

    with Path(final_home_path / 'local_settings.py').open('w') as f:
        f.write('# Only for local test run\n')
        f.write('DEBUG = True\n')
        f.write('SERVE_FILES = True  # May used in urls.py\n')
        f.write('AUTH_PASSWORD_VALIDATORS = []  # accept all passwords\n')

    # call "local_test/manage.py" via subprocess:
    call_manage_py(final_home_path, 'check --deploy')
    if runserver:
        call_manage_py(final_home_path, 'migrate --no-input')
        call_manage_py(final_home_path, 'collectstatic --no-input')
        call_manage_py(final_home_path, 'create_superuser --username="test"')

        os.environ['DJANGO_SETTINGS_MODULE'] = django_settings_name

        # All environment variables are passed to Django's "runnserver" ;)
        # "Simulate" SSOwat authentication, by set "http headers"
        # Still missing is the 'SSOwAuthUser' cookie,
        # but this is ignored, if settings.DEBUG=True ;)
        os.environ['HTTP_AUTH_USER'] = 'test'
        os.environ['HTTP_REMOTE_USER'] = 'test'

        os.environ['HTTP_AUTHORIZATION'] = generate_basic_auth(username='test', password='test123')

        try:
            call_manage_py(final_home_path, 'runserver')
        except KeyboardInterrupt:
            print('\nBye ;)')

    return final_home_path


def cli():
    parser = argparse.ArgumentParser(description='Generate a YunoHost package local test')

    parser.add_argument(
        '--django_settings_path',
        action='store',
        metavar='path',
        help='Path to YunoHost package settings.py file (in "conf" directory)',
    )
    parser.add_argument(
        '--destination',
        action='store',
        metavar='path',
        help='Destination directory for the local test files',
    )
    parser.add_argument(
        '--runserver',
        action='store',
        type=bool,
        default=False,
        help='Start Django "runserver" after local test file creation?',
    )
    args = parser.parse_args()

    create_local_test(
        django_settings_path=Path(args.django_settings_path),
        destination=Path(args.destination),
        runserver=args.runserver,
    )


if __name__ == '__main__':
    cli()



"""
    remote user authentication backend

    Note: SSOwat/nginx add authentication headers:

        'HTTP_AUTHORIZATION': 'Basic XXXXXXXXXXXXXXXX='
        'HTTP_AUTH_USER': 'username'
        'HTTP_REMOTE_USER': 'username'

    Basic auth contains "{username}:{plaintext-password}"

    and we get SSOwat cookies like:

         'HTTP_COOKIE': 'SSOwAuthUser=username; '
                        'SSOwAuthHash=593876aa66...99e69f88af1e; '
                        'SSOwAuthExpire=1609227697.998; '

    * Login a user via HTTP_REMOTE_USER header, but check also username in:
        * SSOwAuthUser
        * HTTP_AUTH_USER
        * HTTP_AUTHORIZATION (Basic auth)
    * Create new users
    * Update Email, First / Last name for existing users
"""

import logging

from django.contrib.auth.backends import RemoteUserBackend

from django_ynh.sso_auth.user_profile import call_setup_user, update_user_profile


logger = logging.getLogger(__name__)


class SSOwatUserBackend(RemoteUserBackend):
    """
    Authentication backend via SSO/nginx header
    """

    create_unknown_user = True

    def authenticate(self, request, remote_user):
        logger.info('Remote user authenticate: %r', remote_user)
        return super().authenticate(request, remote_user)

    def configure_user(self, request, user):
        """
        Configure a new user after creation and return the updated user.
        """
        logger.warning('Configure user %s', user)

        user = update_user_profile(request, user)
        user = call_setup_user(user=user)

        return user

    def user_can_authenticate(self, user):
        logger.warning('Remote user login: %s', user)
        assert not user.is_anonymous
        return True



import logging
import pprint

from django.conf import settings
from django.contrib.auth import get_user_model
from django.http.response import HttpResponse
from django.shortcuts import redirect


logger = logging.getLogger(__name__)


def request_media_debug_view(request):
    """ debug request.META """

    assert settings.DEBUG is True, 'Only in DEBUG mode available!'

    if not request.user.is_authenticated:
        logger.info('Deny debug view: User not logged in!')
        UserModel = get_user_model()
        logger.info('Existing users are: %s', ', '.join(f'"{user}"' for user in UserModel.objects.all()))
        return redirect('admin:index')

    meta = pprint.pformat(request.META)
    html = f'<html><body>request.META: <pre>{meta}</pre></body></html>'

    return HttpResponse(html)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['django_ynh',
 'django_ynh.management',
 'django_ynh.management.commands',
 'django_ynh.sso_auth']

package_data = \
{'': ['*']}

install_requires = \
['django', 'django-axes', 'django-redis', 'gunicorn', 'psycopg2-binary']

setup_kwargs = {
    'name': 'django-ynh',
    'version': '0.1.4',
    'description': 'Glue code to package django projects as yunohost apps.',
    'long_description': '# django_ynh\n\n\nGlue code to package django projects as yunohost apps.\n\nThis repository is:\n\n* The Python package [django-ynh](https://pypi.org/project/django-ynh/) with helpers for integrate a Django project as YunoHost package\n* A example [YunoHost Application](https://install-app.yunohost.org/?app=django_ynh) that can be installed\n\n\n[![Integration level](https://dash.yunohost.org/integration/django_ynh.svg)](https://dash.yunohost.org/appci/app/django_ynh) ![](https://ci-apps.yunohost.org/ci/badges/django_ynh.status.svg) ![](https://ci-apps.yunohost.org/ci/badges/django_ynh.maintain.svg)\n[![Install django_ynh with YunoHost](https://install-app.yunohost.org/install-with-yunohost.svg)](https://install-app.yunohost.org/?app=django_ynh)\n\n\nPull requests welcome ;)\n\n\n## Features\n\n* SSOwat integration (see below)\n* Helper to create first super user for `scripts/install`\n* Run Django development server with a local generated YunoHost package installation (called `local_test`)\n* Run `pytest` against `local_test` "installation"\n\n\n### SSO authentication\n\n[SSOwat](https://github.com/YunoHost/SSOwat) is fully supported:\n\n* First user (`$YNH_APP_ARG_ADMIN`) will be created as Django\'s super user\n* All new users will be created as normal users\n* Login via SSO is fully supported\n* User Email, First / Last name will be updated from SSO data\n\n\n### usage\n\nTo create/update the first user in `install`/`upgrade`, e.g.:\n\n```bash\n./manage.py create_superuser --username="$admin" --email="$admin_mail"\n```\nThis Create/update Django superuser and set a unusable password.\nA password is not needed, because auth done via SSOwat ;)\n\nMain parts in `settings.py`:\n```python\nfrom django_ynh.secret_key import get_or_create_secret as __get_or_create_secret\n\n# Function that will be called to finalize a user profile:\nYNH_SETUP_USER = \'setup_user.setup_project_user\'\n\nSECRET_KEY = __get_or_create_secret(FINAL_HOME_PATH / \'secret.txt\')  # /opt/yunohost/$app/secret.txt\n\nINSTALLED_APPS = [\n    #...\n    \'django_ynh\',\n    #...\n]\n\nMIDDLEWARE = [\n    #... after AuthenticationMiddleware ...\n    #\n    # login a user via HTTP_REMOTE_USER header from SSOwat:\n    \'django_ynh.sso_auth.auth_middleware.SSOwatRemoteUserMiddleware\',\n    #...\n]\n\n# Keep ModelBackend around for per-user permissions and superuser\nAUTHENTICATION_BACKENDS = (\n    \'axes.backends.AxesBackend\',  # AxesBackend should be the first backend!\n    #\n    # Authenticate via SSO and nginx \'HTTP_REMOTE_USER\' header:\n    \'django_ynh.sso_auth.auth_backend.SSOwatUserBackend\',\n    #\n    # Fallback to normal Django model backend:\n    \'django.contrib.auth.backends.ModelBackend\',\n)\n\nLOGIN_REDIRECT_URL = None\nLOGIN_URL = \'/yunohost/sso/\'\nLOGOUT_REDIRECT_URL = \'/yunohost/sso/\'\n```\n\n\n## local test\n\nFor quicker developing of django_ynh in the context of YunoHost app,\nit\'s possible to run the Django developer server with the settings\nand urls made for YunoHost installation.\n\ne.g.:\n```bash\n~$ git clone https://github.com/YunoHost-Apps/django_ynh.git\n~$ cd django_ynh/\n~/django_ynh$ make\ninstall-poetry         install or update poetry\ninstall                install project via poetry\nupdate                 update the sources and installation and generate "conf/requirements.txt"\nlint                   Run code formatters and linter\nfix-code-style         Fix code formatting\ntox-listenvs           List all tox test environments\ntox                    Run pytest via tox with all environments\npytest                 Run pytest\npublish                Release new version to PyPi\nlocal-test             Run local_test.py to run the project locally\nlocal-diff-settings    Run "manage.py diffsettings" with local test\n\n~/django_ynh$ make install-poetry\n~/django_ynh$ make install\n~/django_ynh$ make local-test\n```\n\nNotes:\n\n* SQlite database will be used\n* A super user with username `test` and password `test` is created\n* The page is available under `http://127.0.0.1:8000/app_path/`\n\n\n## history\n\n* [compare v0.1.4...master](https://github.com/YunoHost-Apps/django_ynh/compare/v0.1.4...master) **dev**\n  * tbc\n* [v0.1.4 - 08.01.2021](https://github.com/YunoHost-Apps/django_ynh/compare/v0.1.3...v0.1.4)\n  * Bugfix [CSRF verification failed on POST requests #7](https://github.com/YunoHost-Apps/django_ynh/issues/7)\n* [v0.1.3 - 08.01.2021](https://github.com/YunoHost-Apps/django_ynh/compare/v0.1.2...v0.1.3)\n  * set "DEBUG = True" in local_test (so static files are served and auth works)\n  * Bugfixes and cleanups\n* [v0.1.2 - 29.12.2020](https://github.com/YunoHost-Apps/django_ynh/compare/v0.1.1...v0.1.2)\n  * Bugfixes\n* [v0.1.1 - 29.12.2020](https://github.com/YunoHost-Apps/django_ynh/compare/v0.1.0...v0.1.1)\n  * Refactor "create_superuser" to a manage command, useable via "django_ynh" in `INSTALLED_APPS`\n  * Generate "conf/requirements.txt" and use this file for install\n  * rename own settings and urls (in `/conf/`)\n* [v0.1.0 - 28.12.2020](https://github.com/YunoHost-Apps/django_ynh/compare/f578f14...v0.1.0)\n  * first working state\n* [23.12.2020](https://github.com/YunoHost-Apps/django_ynh/commit/f578f144a3a6d11d7044597c37d550d29c247773)\n  * init the project\n\n\n## Links\n\n* Report a bug about this package: https://github.com/YunoHost-Apps/django_ynh\n* YunoHost website: https://yunohost.org/\n* PyPi package: https://pypi.org/project/django-ynh/\n\nThese projects used `django_ynh`:\n\n* https://github.com/YunoHost-Apps/pyinventory_ynh\n* https://github.com/YunoHost-Apps/django-for-runners_ynh\n\n---\n\n# Developer info\n\n## package installation / debugging\n\nPlease send your pull request to https://github.com/YunoHost-Apps/django_ynh\n\nTry \'main\' branch, e.g.:\n```bash\nsudo yunohost app install https://github.com/YunoHost-Apps/django_ynh/tree/master --debug\nor\nsudo yunohost app upgrade django_ynh -u https://github.com/YunoHost-Apps/django_ynh/tree/master --debug\n```\n\nTry \'testing\' branch, e.g.:\n```bash\nsudo yunohost app install https://github.com/YunoHost-Apps/django_ynh/tree/testing --debug\nor\nsudo yunohost app upgrade django_ynh -u https://github.com/YunoHost-Apps/django_ynh/tree/testing --debug\n```\n\nTo remove call e.g.:\n```bash\nsudo yunohost app remove django_ynh\n```\n\nBackup / remove / restore cycle, e.g.:\n```bash\nyunohost backup create --apps django_ynh\nyunohost backup list\narchives:\n  - django_ynh-pre-upgrade1\n  - 20201223-163434\nyunohost app remove django_ynh\nyunohost backup restore 20201223-163434 --apps django_ynh\n```\n\nDebug installation, e.g.:\n```bash\nroot@yunohost:~# ls -la /var/www/django_ynh/\ntotal 18\ndrwxr-xr-x 4 root root 4 Dec  8 08:36 .\ndrwxr-xr-x 6 root root 6 Dec  8 08:36 ..\ndrwxr-xr-x 2 root root 2 Dec  8 08:36 media\ndrwxr-xr-x 7 root root 8 Dec  8 08:40 static\n\nroot@yunohost:~# ls -la /opt/yunohost/django_ynh/\ntotal 58\ndrwxr-xr-x 5 django_ynh django_ynh   11 Dec  8 08:39 .\ndrwxr-xr-x 3 root        root           3 Dec  8 08:36 ..\n-rw-r--r-- 1 django_ynh django_ynh  460 Dec  8 08:39 gunicorn.conf.py\n-rw-r--r-- 1 django_ynh django_ynh    0 Dec  8 08:39 local_settings.py\n-rwxr-xr-x 1 django_ynh django_ynh  274 Dec  8 08:39 manage.py\n-rw-r--r-- 1 django_ynh django_ynh  171 Dec  8 08:39 secret.txt\ndrwxr-xr-x 6 django_ynh django_ynh    6 Dec  8 08:37 venv\n-rw-r--r-- 1 django_ynh django_ynh  115 Dec  8 08:39 wsgi.py\n-rw-r--r-- 1 django_ynh django_ynh 4737 Dec  8 08:39 django_ynh_demo_settings.py\n\nroot@yunohost:~# cd /opt/yunohost/django_ynh/\nroot@yunohost:/opt/yunohost/django_ynh# source venv/bin/activate\n(venv) root@yunohost:/opt/yunohost/django_ynh# ./manage.py check\ndjango_ynh v0.8.2 (Django v2.2.17)\nDJANGO_SETTINGS_MODULE=\'django_ynh_demo_settings\'\nPROJECT_PATH:/opt/yunohost/django_ynh/venv/lib/python3.7/site-packages\nBASE_PATH:/opt/yunohost/django_ynh\nSystem check identified no issues (0 silenced).\n\nroot@yunohost:~# tail -f /var/log/django_ynh/django_ynh.log\nroot@yunohost:~# cat /etc/systemd/system/django_ynh.service\n\nroot@yunohost:~# systemctl reload-or-restart django_ynh\nroot@yunohost:~# journalctl --unit=django_ynh --follow\n```\n\n\n',
    'author': 'JensDiemer',
    'author_email': 'git@jensdiemer.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/YunoHost-Apps/django_ynh',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0.0',
}


setup(**setup_kwargs)



"""
    Base settings for a Django project installed in Yunohost.
    All values should not depent on YunoHost app settings.
"""


# -----------------------------------------------------------------------------
# settings that should be set in project settings:

SECRET_KEY = None

# -----------------------------------------------------------------------------

ROOT_URLCONF = 'urls'  # .../conf/urls.py

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'axes',  # https://github.com/jazzband/django-axes
    'django_ynh',
]

# -----------------------------------------------------------------------------

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    #
    # login a user via HTTP_REMOTE_USER header from SSOwat:
    'django_ynh.sso_auth.auth_middleware.SSOwatRemoteUserMiddleware',
    #
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    #
    # AxesMiddleware should be the last middleware:
    'axes.middleware.AxesMiddleware',
]

# -----------------------------------------------------------------------------

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

# -----------------------------------------------------------------------------

# Keep ModelBackend around for per-user permissions and superuser
AUTHENTICATION_BACKENDS = (
    'axes.backends.AxesBackend',  # AxesBackend should be the first backend!
    #
    # Authenticate via SSO and nginx 'HTTP_REMOTE_USER' header:
    'django_ynh.sso_auth.auth_backend.SSOwatUserBackend',
    #
    # Fallback to normal Django model backend:
    'django.contrib.auth.backends.ModelBackend',
)

LOGIN_REDIRECT_URL = None
LOGIN_URL = '/yunohost/sso/'
LOGOUT_REDIRECT_URL = '/yunohost/sso/'
# /yunohost/sso/?action=logout

# _____________________________________________________________________________
# Setting below, should be overwritten!

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format': '{asctime} {levelname} {name} {module}.{funcName} {message}',
            'style': '{',
        },
    },
    'handlers': {'console': {'class': 'logging.StreamHandler', 'formatter': 'verbose'}},
    'loggers': {
        'django': {'handlers': ['console'], 'level': 'INFO', 'propagate': False},
        'django.auth': {'handlers': ['console'], 'level': 'DEBUG', 'propagate': False},
        'django.security': {'handlers': ['console'], 'level': 'DEBUG', 'propagate': False},
        'django.request': {'handlers': ['console'], 'level': 'DEBUG', 'propagate': False},
        'django_ynh': {'handlers': ['console'], 'level': 'DEBUG', 'propagate': False},
    },
}
